@@ -74,7 +74,7 @@ BEGIN
   RETURN our_answer;
 
 END;
-$$ LANGUAGE 'plpgsql' IMMUTABLE STRICT;
+$$ LANGUAGE plpgsql IMMUTABLE STRICT;
 
 
 CREATE or REPLACE FUNCTION calculate_later_timestamp( TIMESTAMP WITH TIME ZONE, TIMESTAMP WITH TIME ZONE, TEXT ) RETURNS TIMESTAMP WITH TIME ZONE AS $$
@@ -239,12 +239,12 @@ BEGIN
   RETURN our_answer;
 
 END;
-$$ LANGUAGE 'plpgsql' IMMUTABLE STRICT;
+$$ LANGUAGE plpgsql IMMUTABLE STRICT;
 
 
 CREATE or REPLACE FUNCTION usr_is_role( INT, TEXT ) RETURNS BOOLEAN AS $$
   SELECT EXISTS( SELECT 1 FROM role_member JOIN roles USING(role_no) WHERE role_member.user_no=$1 AND roles.role_name=$2 )
-$$ LANGUAGE 'sql' IMMUTABLE STRICT;
+$$ LANGUAGE sql IMMUTABLE STRICT;
 
 CREATE or REPLACE FUNCTION legacy_get_permissions( INT, INT ) RETURNS TEXT AS $$
 DECLARE
@@ -333,13 +333,13 @@ BEGIN
 
   RETURN '';
 END;
-$$ LANGUAGE 'plpgsql' IMMUTABLE STRICT;
+$$ LANGUAGE plpgsql IMMUTABLE STRICT;
 
 
 -- Function to convert a PostgreSQL date into UTC + the format used by iCalendar
 CREATE or REPLACE FUNCTION to_ical_utc( TIMESTAMP WITH TIME ZONE ) RETURNS TEXT AS $$
   SELECT to_char( $1 at time zone 'UTC', 'YYYYMMDD"T"HH24MISS"Z"' )
-$$ LANGUAGE 'sql' IMMUTABLE STRICT;
+$$ LANGUAGE sql IMMUTABLE STRICT;
 
 -- Function to set an arbitrary DAV property
 CREATE or REPLACE FUNCTION set_dav_property( TEXT, INTEGER, TEXT, TEXT ) RETURNS BOOLEAN AS $$
@@ -365,7 +365,7 @@ BEGIN
   END IF;
   RETURN TRUE;
 END;
-$$ LANGUAGE 'plpgsql' STRICT;
+$$ LANGUAGE plpgsql STRICT;
 
 -- List a user's relationships as a text string
 CREATE or REPLACE FUNCTION relationship_list( INT8 ) RETURNS TEXT AS $$
@@ -385,7 +385,7 @@ BEGIN
   END LOOP;
   RETURN rlist;
 END;
-$$ LANGUAGE 'plpgsql';
+$$ LANGUAGE plpgsql;
 
 DROP FUNCTION rename_davical_user( TEXT, TEXT );
 DROP TRIGGER usr_modified ON usr CASCADE;
@@ -423,12 +423,19 @@ BEGIN
         SET dav_name = replace( dav_name, OLD.dav_name, NEW.dav_name),
             user_no = NEW.user_no
       WHERE substring(dav_name from 1 for char_length(OLD.dav_name)) = OLD.dav_name;
+
+      -- On rename we also need to rename all properties
+      UPDATE property SET dav_name=NEW.dav_name WHERE dav_name=OLD.dav_name;
     END IF;
+    RETURN NEW;
+  ELSIF TG_OP = 'DELETE' THEN
+    -- On delete we remove all properties for the collection
+    DELETE FROM property WHERE dav_name=OLD.dav_name;
+    RETURN OLD;
   END IF;
-  RETURN NEW;
 END;
 $$ LANGUAGE plpgsql;
-CREATE TRIGGER collection_modified AFTER UPDATE ON collection
+CREATE TRIGGER collection_modified AFTER UPDATE OR DELETE ON collection
     FOR EACH ROW EXECUTE PROCEDURE collection_modified();
 
 
@@ -511,7 +518,7 @@ CREATE or REPLACE FUNCTION sync_dav_id ( ) RETURNS TRIGGER AS $$
     RETURN NEW;
 
   END
-$$ LANGUAGE 'plpgsql';
+$$ LANGUAGE plpgsql;
 CREATE TRIGGER caldav_data_sync_dav_id AFTER INSERT OR UPDATE ON caldav_data
     FOR EACH ROW EXECUTE PROCEDURE sync_dav_id();
 
@@ -566,7 +573,7 @@ BEGIN
   RETURN out_bits;
 END
 $$
-LANGUAGE 'PlPgSQL' IMMUTABLE STRICT;
+LANGUAGE plpgsql IMMUTABLE STRICT;
 
 -- This legacy conversion function will eventually be removed, once all logic
 -- has been converted to use bitmaps, or to use the bits_to_priv() output.
@@ -619,7 +626,7 @@ BEGIN
   RETURN out_priv;
 END
 $$
-LANGUAGE 'PlPgSQL' IMMUTABLE STRICT;
+LANGUAGE plpgsql IMMUTABLE STRICT;
 
 CREATE or REPLACE FUNCTION get_permissions( INT, INT ) RETURNS TEXT AS $$
 DECLARE
@@ -672,12 +679,12 @@ BEGIN
 
   RETURN '';
 END;
-$$ LANGUAGE 'plpgsql' IMMUTABLE STRICT;
+$$ LANGUAGE plpgsql IMMUTABLE STRICT;
 
 
 CREATE or REPLACE FUNCTION get_group_role_no() RETURNS INT AS $$
   SELECT role_no FROM roles WHERE role_name = 'Group'
-$$ LANGUAGE 'SQL' IMMUTABLE;
+$$ LANGUAGE sql IMMUTABLE;
 
 CREATE or REPLACE FUNCTION has_legacy_privilege( INT, TEXT, INT ) RETURNS BOOLEAN AS $$
 DECLARE
@@ -726,7 +733,7 @@ BEGIN
 
   RETURN FALSE;
 END;
-$$ LANGUAGE 'plpgsql' IMMUTABLE STRICT;
+$$ LANGUAGE plpgsql IMMUTABLE STRICT;
 
 
 -- Given a verbose DAV: or CalDAV: privilege name return the bitmask
@@ -763,7 +770,7 @@ BEGIN
           ELSE 0 END)::BIT(24);
 END
 $$
-LANGUAGE 'PlPgSQL' IMMUTABLE STRICT;
+LANGUAGE plpgsql IMMUTABLE STRICT;
 
 
 -- Given an array of verbose DAV: or CalDAV: privilege names return the bitmask
@@ -790,7 +797,7 @@ BEGIN
   RETURN out_bits;
 END
 $$
-LANGUAGE 'PlPgSQL' IMMUTABLE STRICT;
+LANGUAGE plpgsql IMMUTABLE STRICT;
 
 
 -- NOTE: Round-trip through this and then back through privilege_to_bits
@@ -885,7 +892,7 @@ BEGIN
   RETURN out_priv;
 END
 $$
-LANGUAGE 'PlPgSQL' IMMUTABLE STRICT;
+LANGUAGE plpgsql IMMUTABLE STRICT;
 
 
 -- Expanded group memberships out to some depth
@@ -895,7 +902,7 @@ CREATE or REPLACE FUNCTION expand_memberships( INT8, INT ) RETURNS SETOF INT8 AS
   SELECT expanded.g_id FROM (SELECT CASE WHEN $2 > 0 THEN expand_memberships( group_id, $2 - 1) END AS g_id
                                FROM group_member WHERE member_id = $1) AS expanded
                        WHERE expanded.g_id IS NOT NULL;
-$$ LANGUAGE 'SQL' STABLE STRICT;
+$$ LANGUAGE sql STABLE STRICT;
 
 -- Expanded group members out to some depth
 CREATE or REPLACE FUNCTION expand_members( INT8, INT ) RETURNS SETOF INT8 AS $$
@@ -904,7 +911,7 @@ CREATE or REPLACE FUNCTION expand_members( INT8, INT ) RETURNS SETOF INT8 AS $$
   SELECT expanded.m_id FROM (SELECT CASE WHEN $2 > 0 THEN expand_members( member_id, $2 - 1) END AS m_id
                                FROM group_member WHERE group_id = $1) AS expanded
                        WHERE expanded.m_id IS NOT NULL;
-$$ LANGUAGE 'SQL' STABLE STRICT;
+$$ LANGUAGE sql STABLE STRICT;
 
 
 
@@ -942,7 +949,7 @@ BEGIN
 
   RETURN out_conferred;
 END;
-$$ LANGUAGE 'plpgsql' STABLE STRICT;
+$$ LANGUAGE plpgsql STABLE STRICT;
 
 
 -- Privileges from accessor to grantor, by user_no
@@ -965,7 +972,7 @@ BEGIN
 
   RETURN out_conferred;
 END;
-$$ LANGUAGE 'plpgsql' STABLE STRICT;
+$$ LANGUAGE plpgsql STABLE STRICT;
 
 
 -- Privileges from accessor (by principal_id) to path
@@ -1017,7 +1024,7 @@ BEGIN
     RETURN ~ out_conferred;
   END IF;
 
-  SELECT privileges INTO out_conferred FROM grants
+  SELECT bit_or(privileges) INTO out_conferred FROM grants
                    WHERE by_collection = grantor_collection
                      AND (to_principal=in_accessor OR to_principal IN (SELECT expand_memberships(in_accessor,in_depth)));
 
@@ -1039,7 +1046,7 @@ BEGIN
 
   RETURN out_conferred;
 END;
-$$ LANGUAGE 'plpgsql' STABLE STRICT;
+$$ LANGUAGE plpgsql STABLE STRICT;
 
 
 -- List a user's memberships as a text string
@@ -1059,7 +1066,7 @@ BEGIN
   END LOOP;
   RETURN mlist;
 END;
-$$ LANGUAGE 'plpgsql' STRICT;
+$$ LANGUAGE plpgsql STRICT;
 
 
 -- List a user's members as a text string
@@ -1079,7 +1086,7 @@ BEGIN
   END LOOP;
   RETURN mlist;
 END;
-$$ LANGUAGE 'plpgsql' STRICT;
+$$ LANGUAGE plpgsql STRICT;
 
 
 -- List the privileges as a text string
@@ -1106,7 +1113,7 @@ BEGIN
   END IF;
   RETURN plist;
 END;
-$$ LANGUAGE 'plpgsql' IMMUTABLE STRICT;
+$$ LANGUAGE plpgsql IMMUTABLE STRICT;
 
 
 DROP TRIGGER principal_modified ON principal CASCADE;
@@ -1168,7 +1175,7 @@ CREATE or REPLACE FUNCTION p_has_proxy_access_to( INT8, INT ) RETURNS SETOF INT8
            WHERE (default_privileges & 5::BIT(24)) != 0::BIT(24)
              AND principal_id != $1
     ) subquery;
-$$ LANGUAGE 'SQL' STABLE STRICT;
+$$ LANGUAGE sql STABLE STRICT;
 
 
 -- A list of the principals who can proxy to this principal
@@ -1178,7 +1185,7 @@ CREATE or REPLACE FUNCTION grants_proxy_access_from_p( INT8, INT ) RETURNS SETOF
    WHERE by_collection IS NULL AND by_principal != $1
      AND by_principal IN (SELECT expand_members(g2.to_principal,$2) FROM grants g2 WHERE g2.by_principal = $1)
    ;
-$$ LANGUAGE 'SQL' STABLE STRICT;
+$$ LANGUAGE sql STABLE STRICT;
 
 
 
@@ -1202,7 +1209,7 @@ BEGIN
                      VALUES( in_collection_id, in_status, tmp_int, in_dav_name);
   RETURN TRUE;
 END
-$$ LANGUAGE 'PlPgSQL' VOLATILE STRICT;
+$$ LANGUAGE plpgsql VOLATILE STRICT;
 
 
 CREATE or REPLACE FUNCTION new_sync_token( INT8, INT8 ) RETURNS INT8 AS $$
@@ -1210,27 +1217,44 @@ DECLARE
   in_old_sync_token ALIAS FOR $1;
   in_collection_id ALIAS FOR $2;
   tmp_int INT8;
+  new_token sync_tokens.sync_token%TYPE;
   old_modification_time sync_tokens.modification_time%TYPE;
 BEGIN
   IF in_old_sync_token > 0 THEN
-    SELECT modification_time INTO old_modification_time FROM sync_tokens WHERE sync_token = in_old_sync_token;
+    SELECT modification_time INTO old_modification_time FROM sync_tokens
+           WHERE sync_token = in_old_sync_token AND collection_id = in_collection_id;
     IF NOT FOUND THEN
       -- They are in an inconsistent state: we return NULL so they can re-start the process
       RETURN NULL;
     END IF;
+  END IF;
+  
+  -- Find the most recent sync_token
+  SELECT sync_token, modification_time INTO new_token, old_modification_time FROM sync_tokens
+         WHERE collection_id = in_collection_id ORDER BY modification_time DESC LIMIT 1;
+  IF FOUND THEN
     SELECT 1 INTO tmp_int FROM sync_changes WHERE collection_id = in_collection_id AND sync_time > old_modification_time LIMIT 1;
     IF NOT FOUND THEN
-      -- Ensure we return the latest sync_token we have for this collection, since there are
-      -- no changes.
-	  SELECT sync_token INTO tmp_int FROM sync_tokens WHERE collection_id = in_collection_id ORDER BY modification_time DESC LIMIT 1;
-      RETURN tmp_int;
+      -- Return the latest sync_token we have for this collection, since there are no changes.
+      RETURN new_token;
     END IF;
   END IF;
-  SELECT nextval('sync_tokens_sync_token_seq') INTO tmp_int;
-  INSERT INTO sync_tokens(collection_id, sync_token) VALUES( in_collection_id, tmp_int );
-  RETURN tmp_int;
+  
+  -- Looks like we need a new sync_token for this collection...
+  SELECT nextval('sync_tokens_sync_token_seq') INTO new_token;
+  INSERT INTO sync_tokens(collection_id, sync_token) VALUES( in_collection_id, new_token );
+  
+  -- Having created our new token we do some clean-up of old tokens
+  SELECT modification_time, sync_token INTO old_modification_time, tmp_int FROM sync_tokens
+  		WHERE collection_id = in_collection_id AND modification_time < (current_timestamp - '7 days'::interval)
+  		ORDER BY collection_id, modification_time DESC;
+  DELETE FROM sync_changes WHERE collection_id = in_collection_id AND sync_time < old_modification_time;
+  DELETE FROM sync_tokens WHERE collection_id = in_collection_id AND sync_token < tmp_int;
+  
+  -- Returning the new token
+  RETURN new_token;
 END
-$$ LANGUAGE 'PlPgSQL' STRICT;
+$$ LANGUAGE plpgsql STRICT;
 
 
 DROP TRIGGER alarm_changed ON calendar_alarm CASCADE;
@@ -1278,4 +1302,55 @@ BEGIN
   RETURN FALSE;
 END;
 $$ LANGUAGE plpgsql ;
-    
\ No newline at end of file
+
+
+
+CREATE or REPLACE FUNCTION collections_within( INT, INT ) RETURNS SETOF INT AS $$
+DECLARE
+    in_collection_id ALIAS FOR $1;
+    in_depth ALIAS FOR $2;
+    resource_id INT;
+    found_some BOOLEAN;
+BEGIN
+    in_depth := in_depth - 1;
+    found_some = FALSE;
+	FOR resource_id IN SELECT b.bound_source_id FROM dav_binding b
+                        JOIN collection pc ON (b.parent_container = pc.dav_name)
+                        WHERE pc.collection_id = in_collection_id
+    LOOP
+        found_some = TRUE;
+        RETURN NEXT resource_id;
+        IF in_depth > 0 THEN
+            FOR resource_id IN SELECT * FROM collections_within( resource_id, in_depth ) LOOP
+                RETURN NEXT resource_id;
+            END LOOP;
+        END IF;
+    END LOOP;
+    FOR resource_id IN SELECT c.collection_id FROM collection c
+                        JOIN collection pc ON (c.parent_container = pc.dav_name)
+                        WHERE pc.collection_id = in_collection_id
+    LOOP
+        found_some = TRUE;
+        RETURN NEXT resource_id;
+        IF in_depth > 0 THEN
+            FOR resource_id IN SELECT * FROM collections_within( resource_id, in_depth ) LOOP
+                RETURN NEXT resource_id;
+            END LOOP;
+        END IF;
+    END LOOP;
+    IF found_some THEN
+        RETURN;
+    END IF;
+    FOR resource_id IN SELECT c.collection_id FROM collection c
+                        JOIN dav_principal pc ON (c.parent_container = pc.dav_name)
+                        WHERE pc.principal_id = in_collection_id
+    LOOP
+        RETURN NEXT resource_id;
+        IF in_depth > 0 THEN
+            FOR resource_id IN SELECT * FROM collections_within( resource_id, in_depth ) LOOP
+                RETURN NEXT resource_id;
+            END LOOP;
+        END IF;
+    END LOOP;
+END;
+$$ LANGUAGE plpgsql ;
