@@ -81,11 +81,13 @@ function send_dav_header() {
     header( 'DAV: '.trim($v, ', '), false);
   }
 }
-send_dav_header();  // Avoid polluting global namespace
 
 require_once('CalDAVRequest.php');
 $request = new CalDAVRequest();
 
+//if ( $request->method == 'OPTIONS' || $c->always_send_dav_header )
+    send_dav_header();  // Avoid polluting global namespace
+
 $allowed = implode( ', ', array_keys($request->supported_methods) );
 // header( 'Allow: '.$allowed);
 
@@ -110,7 +112,7 @@ switch ( $request->method ) {
   case 'HEAD':       include('caldav-GET.php');            break;
   case 'PROPPATCH':  include('caldav-PROPPATCH.php');      break;
   case 'POST':
-    if ( !$add_member ) {
+    if ( $request->content_type != 'text/vcard' && !$add_member ) {
       include('caldav-POST.php');
       break;
     }
@@ -147,5 +149,5 @@ switch ( $request->method ) {
     dbg_error_log( 'caldav', 'RAW: %s', str_replace("\n", '',str_replace("\r", '', $request->raw_post)) );
 }
 
-$request->DoResponse( 500, translate('The application program does not understand that request.') );
+$request->DoResponse( 400, translate('The application program does not understand that request.') );
 
