@@ -17,10 +17,11 @@
 require_once("AwlCache.php");
 require_once("XMLDocument.php");
 require_once("DAVPrincipal.php");
-include("DAVTicket.php");
+require_once("DAVTicket.php");
 
 define('DEPTH_INFINITY', 9999);
 
+
 /**
 * A class for collecting things to do with this request.
 *
@@ -105,6 +106,12 @@ class CalDAVRequest
   */
   public $ticket;
 
+  /**
+   * An array of values from the 'Prefer' header.  At present only 'return-minimal' is acted on in any way - you
+   * can test that value with the PreferMinimal() method.
+   */
+  private $prefer;
+  
   /**
   * Create a new CalDAVRequest object.
   */
@@ -114,6 +121,15 @@ class CalDAVRequest
     $this->options = $options;
     if ( !isset($this->options['allow_by_email']) ) $this->options['allow_by_email'] = false;
 
+    if ( isset($_SERVER['HTTP_PREFER']) ) {
+      $this->prefer = explode( ',', $_SERVER['HTTP_PREFER']);
+    }
+    else if ( isset($_SERVER['HTTP_BRIEF']) && (strtoupper($_SERVER['HTTP_BRIEF']) == 'T') ) {
+      $this->prefer = array( 'return-minimal');
+    }
+    else 
+      $this->prefer = array(); 
+
     /**
     * Our path is /<script name>/<user name>/<user controlled> if it ends in
     * a trailing '/' then it is referring to a DAV 'collection' but otherwise
@@ -127,7 +143,22 @@ class CalDAVRequest
     *  4. otherwise we query the defined relationships between users and use
     *     the minimum privileges returned from that analysis.
     */
-    $this->path = (isset($_SERVER['PATH_INFO']) ? $_SERVER['PATH_INFO'] : "/");
+    if ( isset($_SERVER['PATH_INFO']) ) {
+      $this->path = $_SERVER['PATH_INFO'];
+    }
+    else {
+      $this->path = '/';
+      if ( isset($_SERVER['REQUEST_URI']) ) {
+        if ( preg_match( '{^(.*?\.php)(.*)$}', $_SERVER['REQUEST_URI'], $matches ) ) {
+          $this->path = $matches[2];
+          if ( substr($this->path,0,1) != '/' )
+            $this->path = '/'.$this->path;
+        }
+        else if ( $_SERVER['REQUEST_URI'] != '/' ) {
+          dbg_error_log('LOG', 'Server is not supplying PATH_INFO and REQUEST_URI does not include a PHP program.  Wildly guessing "/"!!!');
+        }
+      }
+    }
     $this->path = rawurldecode($this->path);
 
     /** Allow a request for .../calendar.ics to translate into the calendar URL */
@@ -135,6 +166,10 @@ class CalDAVRequest
       $this->path = $matches[1]. '/';
     }
 
+    if ( isset($c->replace_path) && isset($c->replace_path['from']) && isset($c->replace_path['to']) ) {
+      $this->path = preg_replace($c->replace_path['from'], $c->replace_path['to'], $this->path);
+    }
+
     // dbg_error_log( "caldav", "Sanitising path '%s'", $this->path );
     $bad_chars_regex = '/[\\^\\[\\(\\\\]/';
     if ( preg_match( $bad_chars_regex, $this->path ) ) {
@@ -148,7 +183,7 @@ class CalDAVRequest
       @dbg_error_log('caldav', 'Content-Encoding: %s', $encoding );
       $encoding = preg_replace('{[^a-z0-9-]}i','',$encoding);
       if ( ! ini_get('open_basedir') && (isset($c->dbg['ALL']) || isset($c->dbg['caldav'])) ) {
-        $fh = fopen('/tmp/encoded_data.'.$encoding,'w');
+        $fh = fopen('/var/log/davical/encoded_data.debug'.$encoding,'w');
         if ( $fh ) {
           fwrite($fh,$c->raw_post);
           fclose($fh);
@@ -186,11 +221,11 @@ class CalDAVRequest
     if ( preg_match( '{^(\S+/\S+)\s*(;.*)?$}', $this->content_type, $matches ) ) {
       $this->content_type = $matches[1];
     }
-    if ( isset($_SERVER['CONTENT_LENGTH']) && $_SERVER['CONTENT_LENGTH'] > 7 ) {
+    if ( strlen($c->raw_post) > 0 ) {
       if ( $this->method == 'PROPFIND' || $this->method == 'REPORT' || $this->method == 'PROPPATCH' || $this->method == 'BIND' || $this->method == 'MKTICKET' || $this->method == 'ACL' ) {
         if ( !preg_match( '{^(text|application)/xml$}', $this->content_type ) ) {
           @dbg_error_log( "LOG request", 'Request is "%s" but client set content-type to "%s". Assuming they meant XML!',
-                                                 $request->method, $this->content_type );
+                                                 $this->method, $this->content_type );
           $this->content_type = 'text/xml';
         }
       }
@@ -198,12 +233,8 @@ class CalDAVRequest
         $this->CoerceContentType();
       }
     }
-    else if ( !preg_match( '{^(text|application)/xml$}', $this->content_type ) ) {
-      if ( $this->method == 'GET' || $this->method == 'HEAD' || $this->method == 'OPTIONS' || $this->method == 'MKCALENDAR' || $this->method == 'MKCOL' ) {
-        @dbg_error_log( "LOG request", '%s Request specified %s content type but none is present. Assuming null content-type.',
-            $request->method, $this->content_type );
-        $this->content_type = 'text/plain';
-      }
+    else {
+      $this->content_type = 'text/plain';
     }
     $this->user_agent = ((isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : "Probably Mulberry"));
 
@@ -220,7 +251,6 @@ class CalDAVRequest
       * chosen.
       */
       switch( $this->method ) {
-        case 'PROPFIND':
         case 'DELETE':
         case 'MOVE':
         case 'COPY':
@@ -229,13 +259,17 @@ class CalDAVRequest
           break;
 
         case 'REPORT':
+          $this->depth = 0;
+          break;
+
+        case 'PROPFIND':
         default:
           $this->depth = 0;
       }
     }
-    if ( $this->depth == 'infinity' ) $this->depth = DEPTH_INFINITY;
+    if ( !is_int($this->depth) && "infinity" == $this->depth  ) $this->depth = DEPTH_INFINITY;
     $this->depth = intval($this->depth);
-
+    
     /**
     * MOVE/COPY use a "Destination" header and (optionally) an "Overwrite" one.
     */
@@ -370,7 +404,7 @@ EOSQL;
       $this->collection_path = $this->path;
       $this->_is_principal = true;
     }
-    else if ( preg_match( '#^(/[^/]+)/?$#', $this->path, $matches) || preg_match( '#^(/principals/[^/]+/[^/]+)/?$#', $this->path, $matches) ) {
+    else if ( preg_match( '#^(/[^/?]+)/?$#', $this->path, $matches) || preg_match( '#^(/principals/[^/]+/[^/]+)/?$#', $this->path, $matches) ) {
       $this->collection_id = -1;
       $this->collection_path = $matches[1].'/';  // Enforce trailling '/'
       $this->collection_type = 'principal';
@@ -737,8 +771,9 @@ EOSQL;
   * Returns the tail of a Regex appropriate for this Depth, when appended to
   *
   */
-  function DepthRegexTail() {
+  function DepthRegexTail( $for_collection_report = false) {
     if ( $this->IsInfiniteDepth() ) return '';
+    if ( $this->depth == 0 && $for_collection_report ) return '[^/]+$';
     if ( $this->depth == 0 ) return '$';
     return '[^/]*/?$';
   }
@@ -891,6 +926,17 @@ EOSQL;
   }
 
 
+  /**
+   * Returns true if the 'Prefer: return-minimal' or 'Brief: t' were present in the request headers.
+   */
+  function PreferMinimal() {
+    if ( empty($this->prefer) ) return false;
+    foreach( $this->prefer AS $v ) {
+      if ( $v == 'return-minimal' ) return true;
+    }
+    return false;
+  }
+
   /**
   * Returns true if the URL referenced by this request points at a collection.
   */
@@ -1069,6 +1115,59 @@ EOSQL;
   }
 
 
+  /**
+   * Check that the incoming Etag matches the one for the existing (or non-existing) resource.
+   * 
+   * @param boolean $exists Whether the destination exists
+   * @param string $dest_etag The etag for the destination.
+   */
+  function CheckEtagMatch( $exists, $dest_etag ) {
+    global $c;
+    
+    if ( ! $exists ) {
+      if ( (isset($this->etag_if_match) && $this->etag_if_match != '') ) {
+        /**
+        * RFC2068, 14.25:
+        * If none of the entity tags match, or if "*" is given and no current
+        * entity exists, the server MUST NOT perform the requested method, and
+        * MUST return a 412 (Precondition Failed) response.
+        */
+        $this->PreconditionFailed(412, 'if-match', translate('No resource exists at the destination.'));
+      }
+    }
+    else {
+    
+      if ( isset($c->strict_etag_checking) && $c->strict_etag_checking )
+         $trim_chars = '\'\\" ';
+      else
+        $trim_chars = ' ';
+
+      if ( isset($this->etag_if_match) && $this->etag_if_match != '' && trim( $this->etag_if_match, $trim_chars) != trim( $dest_etag, $trim_chars ) ) {
+        /**
+        * RFC2068, 14.25:
+        * If none of the entity tags match, or if "*" is given and no current
+        * entity exists, the server MUST NOT perform the requested method, and
+        * MUST return a 412 (Precondition Failed) response.
+        */
+        $this->PreconditionFailed(412,'if-match',sprintf('Existing resource ETag of <<%s>> does not match <<%s>>', $dest_etag, $this->etag_if_match) );
+      }
+      else if ( isset($this->etag_none_match) && $this->etag_none_match != ''
+                   && ($this->etag_none_match == $dest_etag || $this->etag_none_match == '*') ) {
+        /**
+        * RFC2068, 14.26:
+        * If any of the entity tags match the entity tag of the entity that
+        * would have been returned in the response to a similar GET request
+        * (without the If-None-Match header) on that resource, or if "*" is
+        * given and any current entity exists for that resource, then the
+        * server MUST NOT perform the requested method.
+        */
+        $this->PreconditionFailed(412,'if-none-match', translate( 'Existing resource matches "If-None-Match" header - not accepted.'));
+      }
+    }
+
+  }
+
+  
   /**
   * Is the user has the privileges to do what is requested.
   */
@@ -1164,11 +1263,15 @@ EOSQL;
   function XMLResponse( $status, $xmltree ) {
     $xmldoc = $xmltree->Render(0,'<?xml version="1.0" encoding="utf-8" ?>');
     $etag = md5($xmldoc);
-    header("ETag: \"$etag\"");
+    if ( !headers_sent() ) header("ETag: \"$etag\"");
     $this->DoResponse( $status, $xmldoc, 'text/xml; charset="utf-8"' );
     exit(0);  // Unecessary, but might clarify things
   }
 
+  public static function kill_on_exit() {
+    posix_kill( getmypid(), 28 );
+  }
+  
   /**
   * Utility function we call when we have a simple status-based response to
   * return to the client.  Possibly
@@ -1178,11 +1281,13 @@ EOSQL;
   */
   function DoResponse( $status, $message="", $content_type="text/plain; charset=\"utf-8\"" ) {
     global $session, $c;
-    @header( sprintf("HTTP/1.1 %d %s", $status, getStatusMessage($status)) );
-    @header( sprintf("X-DAViCal-Version: DAViCal/%d.%d.%d; DB/%d.%d.%d", $c->code_major, $c->code_minor, $c->code_patch, $c->schema_major, $c->schema_minor, $c->schema_patch) );
-    @header( "Content-type: ".$content_type );
+    if ( !headers_sent() ) @header( sprintf("HTTP/1.1 %d %s", $status, getStatusMessage($status)) );
+    if ( !headers_sent() ) @header( sprintf("X-DAViCal-Version: DAViCal/%d.%d.%d; DB/%d.%d.%d", $c->code_major, $c->code_minor, $c->code_patch, $c->schema_major, $c->schema_minor, $c->schema_patch) );
+    if ( !headers_sent() ) header( "Content-type: ".$content_type );
 
-    if ( (isset($c->dbg['ALL']) && $c->dbg['ALL']) || (isset($c->dbg['response']) && $c->dbg['response']) || $status > 399 ) {
+    if ( (isset($c->dbg['ALL']) && $c->dbg['ALL']) || (isset($c->dbg['response']) && $c->dbg['response'])
+         || $status == 400  || $status == 402 || $status == 403 || $status > 404 ) {
+      @dbg_error_log( "LOG ", 'Response status %03d for %s %s', $status, $this->method, $_SERVER['REQUEST_URI'] );
       $lines = headers_list();
       dbg_error_log( "LOG ", "***************** Response Header ****************" );
       foreach( $lines AS $v ) {
@@ -1196,8 +1301,10 @@ EOSQL;
       }
     }
 
-    header( "Content-Length: ".strlen($message) );
-    echo $message;
+    if ( $message != '' ) {
+      if ( !headers_sent() ) header( "Content-Length: ".strlen($message) );
+      echo $message;
+    }
 
     if ( isset($c->dbg['caldav']) && $c->dbg['caldav'] ) {
       if ( strlen($message) > 100 || strstr($message, "\n") ) {
@@ -1208,10 +1315,25 @@ EOSQL;
     }
     if ( isset($c->dbg['statistics']) && $c->dbg['statistics'] ) {
       $script_time = microtime(true) - $c->script_start_time;
-      @dbg_error_log("statistics", "Method: %s, Status: %d, Script: %5.3lfs, Queries: %5.3lfs, URL: %s",
-                         $this->method, $status, $script_time, $c->total_query_time, $this->path);
+      $memory = '';
+      if ( function_exists('memory_get_usage') ) {
+        $memory = sprintf( ', Memory: %dk, Peak: %dk', memory_get_usage()/1024, memory_get_peak_usage(true)/1024);
+      }
+      @dbg_error_log("statistics", "Method: %s, Status: %d, Script: %5.3lfs, Queries: %5.3lfs, URL: %s%s",
+                         $this->method, $status, $script_time, $c->total_query_time, $this->path, $memory);
     }
-    @ob_flush(); exit(0);
+    try {
+      @ob_flush(); // Seems like it should be better to do the following but is problematic on PHP5.3 at least: while ( ob_get_level() > 0 ) ob_end_flush();
+    }
+    catch( Exception $ignored ) {}
+
+    if ( isset($c->exit_after_memory_exceeds) && function_exists('memory_get_peak_usage') && memory_get_peak_usage(true) > $c->exit_after_memory_exceeds ) { // 64M
+      @dbg_error_log("statistics", "Peak memory use exceeds %d bytes (%d) - killing process %d", $c->exit_after_memory_exceeds, memory_get_peak_usage(true), getmypid());
+      register_shutdown_function( 'CalDAVRequest::kill_on_exit' );
+    }
+    
+    
+    exit(0);
   }
 
 }
