@@ -163,7 +163,7 @@ class DAVPrincipal extends Principal
 
     if ( $this->_is_group ) {
       $this->group_member_set = array();
-      $qry = new AwlQuery('SELECT usr.username FROM group_member JOIN principal ON (principal_id=member_id) JOIN usr USING(user_no) WHERE group_id = :group_id ORDER BY principal.principal_id ', array( ':group_id' => $this->principal_id) );
+      $qry = new AwlQuery('SELECT usr.username FROM group_member JOIN principal ON (principal_id=member_id) JOIN usr USING(user_no) WHERE usr.active=true AND group_id = :group_id ORDER BY principal.principal_id ', array( ':group_id' => $this->principal_id) );
       if ( $qry->Exec('DAVPrincipal') && $qry->rows() > 0 ) {
         while( $member = $qry->Fetch() ) {
           $this->group_member_set[] = ConstructURL( '/'. $member->username . '/', true);
@@ -172,7 +172,7 @@ class DAVPrincipal extends Principal
     }
 
     $this->group_membership = array();
-    $qry = new AwlQuery('SELECT usr.username FROM group_member JOIN principal ON (principal_id=group_id) JOIN usr USING(user_no) WHERE member_id = :member_id UNION SELECT usr.username FROM group_member LEFT JOIN grants ON (to_principal=group_id) JOIN principal ON (principal_id=by_principal) JOIN usr USING(user_no) WHERE member_id = :member_id and by_principal != member_id ORDER BY 1', array( ':member_id' => $this->principal_id ) );
+    $qry = new AwlQuery('SELECT usr.username FROM group_member JOIN principal ON (principal_id=group_id) JOIN usr USING(user_no) WHERE usr.active=true AND member_id = :member_id UNION SELECT usr.username FROM group_member LEFT JOIN grants ON (to_principal=group_id) JOIN principal ON (principal_id=by_principal) JOIN usr USING(user_no) WHERE usr.active=true AND member_id = :member_id and by_principal != member_id ORDER BY 1', array( ':member_id' => $this->principal_id ) );
     if ( $qry->Exec('DAVPrincipal') && $qry->rows() > 0 ) {
       while( $group = $qry->Fetch() ) {
         $this->group_membership[] = ConstructURL( '/'. $group->username . '/', true);
@@ -199,44 +199,43 @@ class DAVPrincipal extends Principal
     $this->write_proxy_for = array();
     $this->read_proxy_for = array();
 
-    if ( !isset($c->disable_caldav_proxy) || $c->disable_caldav_proxy === false ) {
+    if ( isset($c->disable_caldav_proxy) && $c->disable_caldav_proxy ) return;
 
-      $write_priv = privilege_to_bits(array('write'));
-      // whom are we a proxy for? who is a proxy for us?
-      // (as per Caldav Proxy section 5.1 Paragraph 7 and 5)
-      $sql = 'SELECT principal_id, username, pprivs(:request_principal::int8,principal_id,:scan_depth::int) FROM principal JOIN usr USING(user_no) WHERE principal_id IN (SELECT * from p_has_proxy_access_to(:request_principal,:scan_depth))';
-      $params = array( ':request_principal' => $this->principal_id, ':scan_depth' => $c->permission_scan_depth );
-      $qry = new AwlQuery($sql, $params);
-      if ( $qry->Exec('DAVPrincipal') && $qry->rows() > 0 ) {
-        while( $relationship = $qry->Fetch() ) {
-          if ( (bindec($relationship->pprivs) & $write_priv) != 0 ) {
-            $this->write_proxy_for[] = ConstructURL( '/'. $relationship->username . '/', true);
-            $this->group_membership[] = ConstructURL( '/'. $relationship->username . '/calendar-proxy-write/', true);
-          }
-          else {
-            $this->read_proxy_for[] = ConstructURL( '/'. $relationship->username . '/', true);
-            $this->group_membership[] = ConstructURL( '/'. $relationship->username . '/calendar-proxy-read/', true);
-          }
+    $write_priv = privilege_to_bits(array('write'));
+    // whom are we a proxy for? who is a proxy for us?
+    // (as per Caldav Proxy section 5.1 Paragraph 7 and 5)
+    $sql = 'SELECT principal_id, username, pprivs(:request_principal::int8,principal_id,:scan_depth::int) FROM principal JOIN usr USING(user_no) WHERE usr.active=true AND principal_id IN (SELECT * from p_has_proxy_access_to(:request_principal,:scan_depth))';
+    $params = array( ':request_principal' => $this->principal_id, ':scan_depth' => $c->permission_scan_depth );
+    $qry = new AwlQuery($sql, $params);
+    if ( $qry->Exec('DAVPrincipal') && $qry->rows() > 0 ) {
+      while( $relationship = $qry->Fetch() ) {
+        if ( (bindec($relationship->pprivs) & $write_priv) != 0 ) {
+          $this->write_proxy_for[] = ConstructURL( '/'. $relationship->username . '/', true);
+          $this->group_membership[] = ConstructURL( '/'. $relationship->username . '/calendar-proxy-write/', true);
+        }
+        else {
+          $this->read_proxy_for[] = ConstructURL( '/'. $relationship->username . '/', true);
+          $this->group_membership[] = ConstructURL( '/'. $relationship->username . '/calendar-proxy-read/', true);
         }
       }
+    }
 
-      $sql = 'SELECT principal_id, username, pprivs(:request_principal::int8,principal_id,:scan_depth::int) FROM principal JOIN usr USING(user_no) WHERE principal_id IN (SELECT * from grants_proxy_access_from_p(:request_principal,:scan_depth))';
-      $qry = new AwlQuery($sql, $params ); // reuse $params assigned for earlier query
-      if ( $qry->Exec('DAVPrincipal') && $qry->rows() > 0 ) {
-        while( $relationship = $qry->Fetch() ) {
-          if ( bindec($relationship->pprivs) & $write_priv ) {
-            $this->write_proxy_group[] = ConstructURL( '/'. $relationship->username . '/', true);
-          }
-          else {
-            $this->read_proxy_group[] = ConstructURL( '/'. $relationship->username . '/', true);
-          }
+    $sql = 'SELECT principal_id, username, pprivs(:request_principal::int8,principal_id,:scan_depth::int) FROM principal JOIN usr USING(user_no) WHERE usr.active=true AND principal_id IN (SELECT * from grants_proxy_access_from_p(:request_principal,:scan_depth))';
+    $qry = new AwlQuery($sql, $params ); // reuse $params assigned for earlier query
+    if ( $qry->Exec('DAVPrincipal') && $qry->rows() > 0 ) {
+      while( $relationship = $qry->Fetch() ) {
+        if ( bindec($relationship->pprivs) & $write_priv ) {
+          $this->write_proxy_group[] = ConstructURL( '/'. $relationship->username . '/', true);
+        }
+        else {
+          $this->read_proxy_group[] = ConstructURL( '/'. $relationship->username . '/', true);
         }
       }
+    }
 //      @dbg_error_log( 'principal', 'Read-proxy-for:    %s', implode(',',$this->read_proxy_for) );
 //      @dbg_error_log( 'principal', 'Write-proxy-for:   %s', implode(',',$this->write_proxy_for) );
 //      @dbg_error_log( 'principal', 'Read-proxy-group:  %s', implode(',',$this->read_proxy_group) );
 //      @dbg_error_log( 'principal', 'Write-proxy-group: %s', implode(',',$this->write_proxy_group) );
-    }
   }
 
 
@@ -406,7 +405,7 @@ class DAVPrincipal extends Principal
     if ( !isset($this->privileges) ) $this->privileges = 0;
     if ( is_string($this->privileges) ) $this->privileges = bindec( $this->privileges );
     if ( $this->_is_group ) {
-      if ( in_array($session->principal->url(), $this->GroupMemberSet()) ) {
+      if ( isset($session->principal) && in_array($session->principal->url(), $this->GroupMemberSet()) ) {
         $this->privileges |=  privilege_to_bits( array('DAV::read', 'DAV::read-current-user-privilege-set') );
       } 
     }
@@ -453,43 +452,45 @@ class DAVPrincipal extends Principal
     dbg_error_log('principal',': RenderAsXML: Principal Property "%s"', $tag );
     switch( $tag ) {
       case 'DAV::getcontenttype':
-        $prop->NewElement('getcontenttype', 'httpd/unix-directory' );
+        $reply->DAVElement( $prop, 'getcontenttype', 'httpd/unix-directory' );
         break;
 
       case 'DAV::resourcetype':
-        $prop->NewElement('resourcetype', array( new XMLElement('principal'), new XMLElement('collection')) );
+        $reply->DAVElement( $prop, 'resourcetype', array( new XMLElement('principal'), new XMLElement('collection')) );
         break;
 
       case 'DAV::displayname':
-        $prop->NewElement('displayname', $this->fullname );
+        $reply->DAVElement( $prop, 'displayname', $this->fullname );
         break;
 
       case 'DAV::principal-URL':
-        $prop->NewElement('principal-URL', $reply->href($this->url()) );
+        $reply->DAVElement( $prop, 'principal-URL', $reply->href($this->url()) );
         break;
 
       case 'DAV::getlastmodified':
-        $prop->NewElement('getlastmodified', ISODateToHTTPDate($this->modified) );
+        $reply->DAVElement( $prop, 'getlastmodified', ISODateToHTTPDate($this->modified) );
         break;
 
       case 'DAV::creationdate':
-        $prop->NewElement('creationdate', DateToISODate($this->created) );
+        $reply->DAVElement( $prop, 'creationdate', DateToISODate($this->created) );
         break;
 
       case 'DAV::getcontentlanguage':
         /** Use the principal's locale by preference, otherwise system default */
         $locale = (isset($c->current_locale) ? $c->current_locale : '');
         if ( isset($this->locale) && $this->locale != '' ) $locale = $this->locale;
-        $prop->NewElement('getcontentlanguage', $locale );
+        $reply->DAVElement( $prop, 'getcontentlanguage', $locale );
         break;
 
+      case 'http://calendarserver.org/ns/:group-member-set':
       case 'DAV::group-member-set':
         if ( ! $this->_is_group ) return false;
-        $prop->NewElement('group-member-set', $reply->href($this->group_member_set) );
+        $reply->DAVElement( $prop, 'group-member-set', $reply->href($this->group_member_set) );
         break;
 
+      case 'http://calendarserver.org/ns/:group-membership':
       case 'DAV::group-membership':
-        $prop->NewElement('group-membership', $reply->href($this->GroupMembership()) );
+        $reply->DAVElement( $prop, 'group-membership', $reply->href($this->GroupMembership()) );
         break;
 
       case 'urn:ietf:params:xml:ns:caldav:schedule-inbox-URL':
@@ -519,11 +520,11 @@ class DAVPrincipal extends Principal
         break;
 
       case 'urn:ietf:params:xml:ns:carddav:addressbook-home-set':
-        $reply->NSElement($prop, $tag, $reply->href( $this->addressbook_home_set() ) );
+        $reply->CardDAVElement($prop, $tag, $reply->href( $this->addressbook_home_set() ) );
         break;
 
       case 'urn:ietf:params:xml:ns:caldav:calendar-home-set':
-        $reply->NSElement($prop, $tag, $reply->href( $this->calendar_home_set() ) );
+        $reply->CalDAVElement($prop, $tag, $reply->href( $this->calendar_home_set() ) );
         break;
 
       case 'urn:ietf:params:xml:ns:caldav:calendar-free-busy-set':
@@ -541,7 +542,7 @@ class DAVPrincipal extends Principal
 
       // Empty tag responses.
       case 'DAV::alternate-URI-set':
-        $prop->NewElement( $reply->Tag($tag));
+        $reply->DAVElement( $prop, $reply->Tag($tag));
         break;
 
       case 'SOME-DENIED-PROPERTY':  /** @todo indicating the style for future expansion */
