@@ -52,7 +52,7 @@ class WritableCollection extends DAVResource {
       $resource_type = $first->GetType();
     }
   
-    $uid = $first->GetPValue('UID');
+    $uid = $vcal->GetUID();
     if ( empty($segment_name) ) {
       $segment_name = $uid.'.ics';
     }
@@ -157,7 +157,7 @@ class WritableCollection extends DAVResource {
         * So we're looking for 'VALUE=DATE', to identify the duration, effectively.
         *
         */
-        $value_type = $first->GetPParamValue('DTSTART','VALUE');
+        $value_type = $first->GetProperty('DTSTART')->GetParameterValue('VALUE');
         dbg_error_log('PUT','DTSTART without DTEND. DTSTART value type is %s', $value_type );
         if ( isset($value_type) && $value_type == 'DATE' )
           $dtend = '(:dtstart::timestamp with time zone::date + \'1 day\'::interval)';
@@ -245,6 +245,7 @@ EOSQL;
     if ( !$this->IsSchedulingCollection() ) {
       $this->WriteCalendarAlarms($dav_id, $vcal);
       $this->WriteCalendarAttendees($dav_id, $vcal);
+      $put_action_type = ($create_resource ? 'INSERT' : 'UPDATE');
       if ( $log_action && function_exists('log_caldav_action') ) {
         log_caldav_action( $put_action_type, $first->GetPValue('UID'), $user_no, $collection_id, $path );
       }
@@ -451,7 +452,8 @@ EOSQL;
     $params = array( ':collection_id' => $this->collection_id() );
     if ( $some_old_token == 0 || empty($some_old_token) ) {
       $sql = <<<EOSQL
-  SELECT calendar_item.*, caldav_data.*, addressbook_resource.*, 201 AS sync_status
+  SELECT calendar_item.*, caldav_data.*, addressbook_resource.*, 201 AS sync_status,
+         COALESCE(addressbook_resource.uid,calendar_item.uid) AS uid
       FROM caldav_data
       LEFT JOIN calendar_item USING (dav_id)
       LEFT JOIN addressbook_resource USING (dav_id)
@@ -462,7 +464,8 @@ EOSQL;
     else {
       $params[':sync_token'] = $some_old_token;
       $sql = <<<EOSQL
-  SELECT calendar_item.*, caldav_data.*, addressbook_resource.*, sync_changes.*
+  SELECT calendar_item.*, caldav_data.*, addressbook_resource.*, sync_changes.*,
+         COALESCE(addressbook_resource.uid,calendar_item.uid) AS uid
       FROM sync_changes
       LEFT JOIN caldav_data USING (collection_id,dav_id)
       LEFT JOIN calendar_item USING (collection_id,dav_id)
@@ -476,7 +479,7 @@ EOSQL;
     $qry = new AwlQuery($sql, $params );
 
     $changes = array();
-    if ( $qry->Exec() && $qry->rows() ) {
+    if ( $qry->Exec('WritableCollection') && $qry->rows() ) {
       while( $change = $qry->Fetch() ) {
         $changes[$change->uid] = $change;
       }
