@@ -13,11 +13,13 @@
 require_once('XMLDocument.php');
 
 /**
-* A class for holding basic calendar information
-* @package awl
-*/
+ * A class for holding basic calendar information
+ * @package awl
+ */
 class CalendarInfo {
-  public $url, $displayname, $getctag;
+  public $url;
+  public $displayname;
+  public $getctag;
 
   function __construct( $url, $displayname = null, $getctag = null ) {
     $this->url = $url;
@@ -30,6 +32,9 @@ class CalendarInfo {
   }
 }
 
+if(!defined("_FSOCK_TIMEOUT")){
+  define("_FSOCK_TIMEOUT", 10);
+}
 
 /**
 * A class for accessing DAViCal via CalDAV, as a client
@@ -76,10 +81,15 @@ class CalDAVClient {
   protected $requestMethod = "GET";
   protected $httpRequest = "";  // for debugging http headers sent
   protected $xmlRequest = "";   // for debugging xml sent
-  protected $httpResponse = ""; // http headers received
   protected $xmlResponse = "";  // xml received
+  protected $httpResponseCode = 0; // http response code
+  protected $httpResponseHeaders = "";
+  protected $httpParsedHeaders;
+  protected $httpResponseBody = "";  
 
   protected $parser; // our XML parser object
+  
+  private $debug = false; // Whether we are debugging
 
   /**
   * Constructor, initialises the class
@@ -113,6 +123,23 @@ class CalDAVClient {
     }
   }
 
+  
+  /**
+   * Call this to enable / disable debugging.  It will return the prior value of the debugging flag.
+   * @param boolean $new_value The new value for debugging.
+   * @return boolean The previous value, in case you want to restore it later.
+   */
+  function SetDebug( $new_value ) {
+    $old_value = $this->debug;
+    if ( $new_value )
+      $this->debug = true;
+    else
+      $this->debug = false;
+    return $old_value;
+  }
+
+  
+  
   /**
   * Adds an If-Match or If-None-Match header
   *
@@ -120,7 +147,7 @@ class CalDAVClient {
   * @param string $etag The etag to match / not match against.
   */
   function SetMatch( $match, $etag = '*' ) {
-    $this->headers['match'] = sprintf( "%s-Match: %s", ($match ? "If" : "If-None"), $etag);
+    $this->headers['match'] = sprintf( "%s-Match: \"%s\"", ($match ? "If" : "If-None"), trim($etag,'"'));
   }
 
   /**
@@ -167,11 +194,7 @@ class CalDAVClient {
    */
   function ParseResponse( $response ) {
     $pos = strpos($response, '<?xml');
-    if ($pos === false) {
-      $this->httpResponse = trim($response);
-    }
-    else {
-      $this->httpResponse = trim(substr($response, 0, $pos));
+    if ($pos !== false) {
       $this->xmlResponse = trim(substr($response, $pos));
       $this->xmlResponse = preg_replace('{>[^>]*$}s', '>',$this->xmlResponse );
       $parser = xml_parser_create_ns('UTF-8');
@@ -190,6 +213,31 @@ class CalDAVClient {
     }
   }
 
+  /**
+  * Split httpResponseHeaders into an array of headers
+  *
+  * @return array of arrays of header lines
+   */
+  function ParseResponseHeaders() {
+    if ( empty($this->httpResponseHeaders) ) return array();
+    if ( !isset($this->httpParsedHeaders) ) {
+      $this->httpParsedHeaders = array();
+      $headers = str_replace("\r\n", "\n", $this->httpResponseHeaders);
+      $ar_headers = explode("\n", $headers);
+      $last_header = '';
+      foreach ($ar_headers as $cur_headers) {
+        if( preg_match( '{^\s*\S}', $cur_headers) )  $header_name = $last_header;
+        else if ( preg_match( '{^(\S*):', $cur_headers, $matches) ) {
+          $header_name = $matches[1];
+          $last_header = $header_name;
+          if ( empty($this->httpParsedHeaders[$header_name]) ) $this->httpParsedHeaders[$header_name] = array();
+        }
+        $this->httpParsedHeaders[$header_name][] = $cur_headers;
+      }
+    }
+    return $this->httpParsedHeaders;
+  }
+
   /**
    * Output http request headers
    *
@@ -239,7 +287,6 @@ class CalDAVClient {
   * @return string The content of the response from the server
   */
   function DoRequest( $url = null ) {
-    if(!defined("_FSOCK_TIMEOUT")){ define("_FSOCK_TIMEOUT", 10); }
     $headers = array();
 
     if ( !isset($url) ) $url = $this->base_url;
@@ -268,7 +315,6 @@ class CalDAVClient {
     $this->httpRequest = join("\r\n",$headers);
     $this->xmlRequest = $this->body;
 
-    $this->httpResponse = '';
     $this->xmlResponse = '';
 
     $fip = fsockopen( $this->protocol . '://' . $this->server, $this->port, $errno, $errstr, _FSOCK_TIMEOUT); //error handling?
@@ -280,6 +326,10 @@ class CalDAVClient {
 
     list( $this->httpResponseHeaders, $this->httpResponseBody ) = preg_split( '{\r?\n\r?\n}s', $response, 2 );
     if ( preg_match( '{Transfer-Encoding: chunked}i', $this->httpResponseHeaders ) ) $this->Unchunk();
+    if ( preg_match('/HTTP\/\d\.\d (\d{3})/', $this->httpResponseHeaders, $status) )
+      $this->httpResponseCode = intval($status[1]);
+    else
+      $this->httpResponseCode = 0;
 
     $this->headers = array();  // reset the headers array for our next request
     $this->ParseResponse($this->httpResponseBody);
@@ -328,9 +378,13 @@ class CalDAVClient {
   function DoOptionsRequest( $url = null ) {
     $this->requestMethod = "OPTIONS";
     $this->body = "";
-    $headers = $this->DoRequest($url);
-    $options_header = preg_replace( '/^.*Allow: ([a-z, ]+)\r?\n.*/is', '$1', $headers );
-    $options = array_flip( preg_split( '/[, ]+/', $options_header ));
+    $this->DoRequest($url);
+    $this->ParseResponseHeaders();
+    $allowed = '';
+    foreach( $this->httpParsedHeaders['Allow'] as $allow_header ) {
+      $allowed .= preg_replace( '/^(Allow:)?\s+([a-z, ]+)\r?\n.*/is', '$1,', $allow_header );
+    }
+    $options = array_flip( preg_split( '/[, ]+/', trim($allowed, ', ') ));
     return $options;
   }
 
@@ -394,19 +448,19 @@ class CalDAVClient {
     if ( $etag != null ) {
       $this->SetMatch( ($etag != '*'), $etag );
     }
-    $this->SetContentType('text/calendar; encoding="utf-8"');
+    $this->SetContentType('text/calendar; charset="utf-8"');
     $this->DoRequest($url);
 
     $etag = null;
     if ( preg_match( '{^ETag:\s+"([^"]*)"\s*$}im', $this->httpResponseHeaders, $matches ) ) $etag = $matches[1];
     if ( !isset($etag) || $etag == '' ) {
-      printf( "No etag in:\n%s\n", $this->httpResponseHeaders );
+      if ( $this->debug ) printf( "No etag in:\n%s\n", $this->httpResponseHeaders );
       $save_request = $this->httpRequest;
       $save_response_headers = $this->httpResponseHeaders;
       $this->DoHEADRequest( $url );
       if ( preg_match( '{^Etag:\s+"([^"]*)"\s*$}im', $this->httpResponseHeaders, $matches ) ) $etag = $matches[1];
       if ( !isset($etag) || $etag == '' ) {
-        printf( "Still No etag in:\n%s\n", $this->httpResponseHeaders );
+        if ( $this->debug ) printf( "Still No etag in:\n%s\n", $this->httpResponseHeaders );
       }
       $this->httpRequest = $save_request;
       $this->httpResponseHeaders = $save_response_headers;
@@ -431,7 +485,7 @@ class CalDAVClient {
       $this->SetMatch( true, $etag );
     }
     $this->DoRequest($url);
-    return $this->resultcode;
+    return $this->httpResponseCode;
   }
 
 
@@ -534,7 +588,7 @@ class CalDAVClient {
       }
     }
     else {
-      printf( "xmltags[$tagname] or xmltags[$tagname][$i] is not set\n");
+      if ( $this->debug ) printf( "xmltags[$tagname] or xmltags[$tagname][$i] is not set\n");
     }
     return null;
   }
@@ -599,7 +653,7 @@ class CalDAVClient {
   *
   * @param string $url The URL to find the principal-URL from
   */
-  function FindPrincipal( $url ) {
+  function FindPrincipal( $url=null ) {
     $xml = $this->DoPROPFINDRequest( $url, array('resourcetype', 'current-user-principal', 'owner', 'principal-URL',
                                   'urn:ietf:params:xml:ns:caldav:calendar-home-set'), 1);
 
@@ -925,16 +979,17 @@ EOFILTER;
   *
   * @param uid
   * @param string    $relative_url The URL relative to the base_url specified when the calendar was opened.  Default ''.
+  * @param string    $component_type The component type inside the VCALENDAR.  Default 'VEVENT'.
   *
   * @return array An array of the relative URL, etag, and calendar data returned from DoCalendarQuery() @see DoCalendarQuery()
   */
-  function GetEntryByUid( $uid, $relative_url = '' ) {
+  function GetEntryByUid( $uid, $relative_url = '', $component_type = 'VEVENT' ) {
     $filter = "";
     if ( $uid ) {
       $filter = <<<EOFILTER
   <C:filter>
     <C:comp-filter name="VCALENDAR">
-          <C:comp-filter name="VEVENT">
+          <C:comp-filter name="$component_type">
                 <C:prop-filter name="UID">
                         <C:text-match icollation="i;octet">$uid</C:text-match>
                 </C:prop-filter>
@@ -961,75 +1016,3 @@ EOFILTER;
   }
 
 }
-
-/**
-* Usage example
-*
-* $cal = new CalDAVClient( "http://calendar.example.com/caldav.php/username/calendar/", "username", "password", "calendar" );
-* $options = $cal->DoOptionsRequest();
-* if ( isset($options["PROPFIND"]) ) {
-*   // Fetch some information about the events in that calendar
-*   $cal->SetDepth(1);
-*   $folder_xml = $cal->DoXMLRequest("PROPFIND", '<?xml version="1.0" encoding="utf-8" ?><propfind xmlns="DAV:"><prop><getcontentlength/><getcontenttype/><resourcetype/><getetag/></prop></propfind>' );
-* }
-* // Fetch all events for February
-* $events = $cal->GetEvents("20070101T000000Z","20070201T000000Z");
-* foreach ( $events AS $k => $event ) {
-*   do_something_with_event_data( $event['data'] );
-* }
-* $acc = array();
-* $acc["google"] = array(
-* "user"=>"kunsttherapie@gmail.com",
-* "pass"=>"xxxxx",
-* "server"=>"ssl://www.google.com",
-* "port"=>"443",
-* "uri"=>"https://www.google.com/calendar/dav/kunsttherapie@gmail.com/events/",
-* );
-*
-* $acc["davical"] = array(
-* "user"=>"some_user",
-* "pass"=>"big secret",
-* "server"=>"calendar.foo.bar",
-* "port"=>"80",
-* "uri"=>"http://calendar.foo.bar/caldav.php/some_user/home/",
-* );
-* //*******************************
-*
-* $account = $acc["davical"];
-*
-* //*******************************
-* $cal = new CalDAVClient( $account["uri"], $account["user"], $account["pass"], "", $account["server"], $account["port"] );
-* $options = $cal->DoOptionsRequest();
-* print_r($options);
-*
-* //*******************************
-* //*******************************
-*
-* $xmlC = <<<PROPP
-* <?xml version="1.0" encoding="utf-8" ?>
-* <D:propfind xmlns:D="DAV:" xmlns:C="http://calendarserver.org/ns/">
-*     <D:prop>
-*             <D:displayname />
-*             <C:getctag />
-*             <D:resourcetype />
-*
-*     </D:prop>
-* </D:propfind>
-* PROPP;
-* //if ( isset($options["PROPFIND"]) ) {
-*   // Fetch some information about the events in that calendar
-* //  $cal->SetDepth(1);
-* //  $folder_xml = $cal->DoXMLRequest("PROPFIND", $xmlC);
-* //  print_r( $folder_xml);
-* //}
-*
-* // Fetch all events for February
-* $events = $cal->GetEvents("20090201T000000Z","20090301T000000Z");
-* foreach ( $events as $k => $event ) {
-*     print_r($event['data']);
-*     print "\n---------------------------------------------\n";
-* }
-*
-* //*******************************
-* //*******************************
-*/
