@@ -48,6 +48,8 @@ class ldapDrivers
           ldap_set_option($this->connect, LDAP_OPT_PROTOCOL_VERSION, $config['protocolVersion']);
       if (isset($config['optReferrals']))
           ldap_set_option($this->connect, LDAP_OPT_REFERRALS, $config['optReferrals']);
+      if (isset($config['networkTimeout']))
+          ldap_set_option($this->connect, LDAP_OPT_NETWORK_TIMEOUT, $config['networkTimeout']);
 
       if ($port)
           $this->connect=ldap_connect($host, $port);
@@ -117,6 +119,7 @@ class ldapDrivers
     global $c;
 
     $query = $this->ldap_query_all;
+    $ret = array();
 
     foreach($this->baseDNUsers as $baseDNUsers) {
       $entry = $query($this->connect,$baseDNUsers,$this->filterUsers,$attributes);
@@ -148,6 +151,7 @@ class ldapDrivers
     global $c;
 
     $query = $this->ldap_query_all;
+    $ret = array();
 
     foreach($this->baseDNGroups as $baseDNGroups) {
       $entry = $query($this->connect,$baseDNGroups,$this->filterGroups,$attributes);
@@ -205,9 +209,16 @@ class ldapDrivers
     $dnUser = ldap_get_dn($this->connect, ldap_first_entry($this->connect,$entry));
 
     if ( isset($c->authenticate_hook['config']['i_use_mode_kerberos']) && $c->authenticate_hook['config']['i_use_mode_kerberos'] == "i_know_what_i_am_doing") {
-    	dbg_error_log( "LOG", "drivers_ldap : Skipping password Check for user %s which should be the same as %s",$username , $_SERVER["REMOTE_USER"]);
-      if ($username != $_SERVER["REMOTE_USER"]) {
-        return false;
+      if (isset($_SERVER["REMOTE_USER"])) {
+        dbg_error_log( "LOG", "drivers_ldap : Skipping password Check for user %s which should be the same as %s",$username , $_SERVER["REMOTE_USER"]);
+        if ($username != $_SERVER["REMOTE_USER"]) {
+          return false;
+        }
+      } else {
+        dbg_error_log( "LOG", "drivers_ldap : Skipping password Check for user %s which should be the same as %s",$username , $_SERVER["REDIRECT_REMOTE_USER"]);
+        if ($username != $_SERVER["REDIRECT_REMOTE_USER"]) {
+          return false;
+        }
       }
     }
     else if ( empty($passwd) || preg_match('/[\x00-\x19]/',$passwd) ) {
@@ -222,8 +233,8 @@ class ldapDrivers
       }
     }
 
-
-    dbg_error_log( "LDAP", "drivers_ldap : Bound to user %s using password %s", $dnUser, $passwd );
+    dbg_error_log( "LDAP", "drivers_ldap : Bound to user %s using password %s", $dnUser,
+          (isset($c->dbg['password']) && $c->dbg['password'] ? $passwd : 'another delicious password for the debugging monster!') );
 
     $i = ldap_first_entry($this->connect,$entry);
     $arr = ldap_get_attributes($this->connect,$i);
@@ -247,6 +258,13 @@ function getStaticLdap() {
   // If the instance is not there, create one
   if(!isset($instance)) {
     $ldapDrivers = new ldapDrivers($c->authenticate_hook['config']);
+
+    if ($ldapDrivers->valid) {
+        $instance = $ldapDrivers;
+    }
+  }
+  else {
+      $ldapDrivers = $instance;
   }
   return $ldapDrivers;
 }
@@ -264,9 +282,19 @@ function sync_user_from_LDAP( Principal &$principal, $mapping, $ldap_values ) {
   $fields_to_set = array();
   $updateable_fields = Principal::updateableFields();
   foreach( $updateable_fields AS $field ) {
-    if ( isset($mapping[$field]) && isset($ldap_values[$mapping[$field]]) ) {
-      $fields_to_set[$field] = $ldap_values[$mapping[$field]];
-      dbg_error_log( "LDAP", "Setting usr->%s to %s from LDAP field %s", $field, $ldap_values[$mapping[$field]], $mapping[$field] );
+    if ( isset($mapping[$field]) ) {
+      $tab_part_fields = explode(',',$mapping[$field]);
+      foreach( $tab_part_fields as $part_field ) {
+        if ( isset($ldap_values[$part_field]) ) {
+          if (isset($fields_to_set[$field]) ) {
+            $fields_to_set[$field] .= ' '.$ldap_values[$part_field];
+          }
+          else {
+            $fields_to_set[$field] = $ldap_values[$part_field];
+          }
+        }
+      }
+      dbg_error_log( "LDAP", "Setting usr->%s to %s from LDAP field %s", $field, $fields_to_set[$field], $mapping[$field] );
     }
     else if ( isset($c->authenticate_hook['config']['default_value']) && is_array($c->authenticate_hook['config']['default_value'])
               && isset($c->authenticate_hook['config']['default_value'][$field] ) ) {
@@ -285,6 +313,19 @@ function sync_user_from_LDAP( Principal &$principal, $mapping, $ldap_values ) {
   }
 }
 
+/*
+* explode the multipart mapping
+*/
+function array_values_mapping($mapping){
+  $attributes=array();
+  foreach ( $mapping as $field ) {
+    $tab_part_field = explode(",",$field);
+    foreach( $tab_part_field as $part_field ) {
+      $attributes[] = $part_field;
+    }
+  }
+  return $attributes;
+}
 
 /**
 * Check the username / password against the LDAP server
@@ -294,8 +335,16 @@ function LDAP_check($username, $password ){
 
   $ldapDriver = getStaticLdap();
   if ( !$ldapDriver->valid ) {
-    dbg_error_log( "ERROR", "Couldn't contact LDAP server for authentication" );
-    return false;
+    sleep(1);  // Sleep very briefly to try and survive intermittent issues
+    $ldapDriver = getStaticLdap();
+    if ( !$ldapDriver->valid ) {
+      dbg_error_log( "ERROR", "Couldn't contact LDAP server for authentication" );
+      foreach($c->messages as $msg) {
+          dbg_error_log( "ERROR", "-> ".$msg );
+      }
+      header( sprintf("HTTP/1.1 %d %s", 503, translate("Authentication server unavailable.")) );
+      exit(0);
+    }
   }
 
   $mapping = $c->authenticate_hook['config']['mapping_field'];
@@ -309,7 +358,7 @@ function LDAP_check($username, $password ){
     $mapping['modified'] = $mapping['updated'];
     unset($mapping['updated']);
   }
-  $attributes = array_values($mapping);
+  $attributes = array_values_mapping($mapping);
 
   /**
   * If the config contains a filter that starts with a ( then believe
@@ -365,6 +414,16 @@ function LDAP_check($username, $password ){
 
 }
 
+function fix_unique_member($list) {
+  $fixed_list = array();
+  foreach ( $list as $member ){
+    list( $mem, $rest ) = explode(",", $member );
+    $member = str_replace( 'uid=', '', $mem );
+    array_unshift( $fixed_list, $member );
+  }
+  return $fixed_list;
+}
+
 /**
 * sync LDAP Groups against the DB
 */
@@ -375,7 +434,7 @@ function sync_LDAP_groups(){
 
   $mapping = $c->authenticate_hook['config']['group_mapping_field'];
   //$attributes = array('cn','modifyTimestamp','memberUid');
-  $attributes = array_values($mapping);
+  $attributes = array_values_mapping($mapping);
   $ldap_groups_tmp = $ldapDriver->getAllGroups($attributes);
 
   if ( sizeof($ldap_groups_tmp) == 0 ) return;
@@ -455,6 +514,10 @@ function sync_LDAP_groups(){
       Principal::cacheDelete('username', $group);
       $c->messages[] = sprintf(i18n('- adding users %s to group : %s'),join(',',$ldap_groups_info[$group][$mapping['members']]),$group);
       foreach ( $ldap_groups_info[$group][$mapping['members']] as $member ){
+        if ( $member_field == 'uniqueMember' ) {
+          list( $mem, $rest ) = explode(",", $member );
+          $member = str_replace( 'uid=', '', $mem );
+        }
         $qry = new AwlQuery( "INSERT INTO group_member SELECT g.principal_id AS group_id,u.principal_id AS member_id FROM dav_principal g, dav_principal u WHERE g.username=:group AND u.username=:member;",array (':group'=>$group,':member'=>$member) );
         $qry->Exec('sync_LDAP_groups',__LINE__,__FILE__);
         Principal::cacheDelete('username', $member);
@@ -467,6 +530,9 @@ function sync_LDAP_groups(){
     foreach ( $groups_to_update as $group ){
       $db_members = array_values ( $db_group_members[$group] );
       $ldap_members = array_values ( $ldap_groups_info[$group][$member_field] );
+      if ( $member_field == 'uniqueMember' ) {
+        $ldap_members = fix_unique_member( $ldap_members );
+      }
       $add_users = array_diff ( $ldap_members, $db_members );
       if ( sizeof ( $add_users ) ){
         $c->messages[] = sprintf(i18n('- adding %s to group : %s'),join(', ', $add_users ), $group);
@@ -491,7 +557,7 @@ function sync_LDAP_groups(){
   if ( sizeof ( $groups_to_deactivate ) ){
     $c->messages[] = sprintf(i18n('- deactivate groups : %s'),join(', ',$groups_to_deactivate));
     foreach ( $groups_to_deactivate as $group ){
-      $qry = new AwlQuery( 'UPDATE dav_principal set active=FALSE WHERE username=:group AND type_id = 3',array(':group'=>$group) );
+      $qry = new AwlQuery( 'UPDATE dav_principal SET user_active=FALSE WHERE username=:group AND type_id = 3',array(':group'=>$group) );
       $qry->Exec('sync_LDAP',__LINE__,__FILE__);
       Principal::cacheFlush('username=:group AND type_id = 3', array(':group'=>$group) );
     }
@@ -508,7 +574,7 @@ function sync_LDAP(){
   if ( ! $ldapDriver->valid ) return;
 
   $mapping = $c->authenticate_hook['config']['mapping_field'];
-  $attributes = array_values($mapping);
+  $attributes = array_values_mapping($mapping);
   $ldap_users_tmp = $ldapDriver->getAllUsers($attributes);
 
   if ( sizeof($ldap_users_tmp) == 0 ) return;
@@ -565,16 +631,19 @@ function sync_LDAP(){
   // deactivating all users
   $params = array();
   $i = 0;
+  $paramstring = '';
   foreach( $users_to_deactivate AS $v ) {
     if ( isset($c->do_not_sync_from_ldap) && isset($c->do_not_sync_from_ldap[$v]) ) continue;
+    if ( $i > 0 ) $paramstring .= ',';
+    $paramstring .= ':u'.$i.'::text';
     $params[':u'.$i++] = strtolower($v);
   }
   if ( count($params) > 0 ) {
     $c->messages[] = sprintf(i18n('- deactivating users : %s'),join(', ',$users_to_deactivate));
-    $qry = new AwlQuery( 'UPDATE usr SET active = FALSE WHERE lower(username) IN ('.implode(',',array_keys($params)).')', $params);
+    $qry = new AwlQuery( 'UPDATE usr SET active = FALSE WHERE lower(username) IN ('.$paramstring.')', $params);
     $qry->Exec('sync_LDAP',__LINE__,__FILE__);
 
-    Principal::cacheFlush('lower(username) IN ('.implode(',',array_keys($params)).')', $params);
+    Principal::cacheFlush('lower(username) IN ('.$paramstring.')', $params);
   }
 
   // updating all users
